@@ -190,10 +190,10 @@ ifcore_status(lua_State *lua)
 		return 0;
 
 	lua_pushboolean(lua, (r == STATUS_OK));
-	lua_pushnumber(lua, (lua_Number) (exists));
-	lua_pushnumber(lua, (lua_Number) (recent));
-	lua_pushnumber(lua, (lua_Number) (unseen));
-	lua_pushnumber(lua, (lua_Number) (uidnext));
+	lua_pushinteger(lua, (lua_Integer) (exists));
+	lua_pushinteger(lua, (lua_Integer) (recent));
+	lua_pushinteger(lua, (lua_Integer) (unseen));
+	lua_pushinteger(lua, (lua_Integer) (uidnext));
 
 	return 5;
 }
@@ -975,13 +975,19 @@ static int
 ifcore_idle(lua_State *lua)
 {
 	int r;
+	char *event;
+
+	event = NULL;
 
 	if (lua_gettop(lua) != 1)
 		luaL_error(lua, "wrong number of arguments");
 	luaL_checktype(lua, 1, LUA_TLIGHTUSERDATA);
 
-	while ((r = request_idle((session *)(lua_topointer(lua, 1)))) ==
-	    STATUS_NONE);
+	if (get_option_boolean("reenter"))
+		while ((r = request_idle((session *)(lua_topointer(lua, 1)),
+		    &event)) == STATUS_NONE);
+	else
+		r = request_idle((session *)(lua_topointer(lua, 1)), &event);
 
 	lua_pop(lua, 1);
 
@@ -990,7 +996,14 @@ ifcore_idle(lua_State *lua)
 
 	lua_pushboolean(lua, (r == STATUS_OK));
 
-	return 1;
+	if (!event)
+		return 1;
+
+	lua_pushstring(lua, event);
+
+	xfree(event);
+
+	return 2;
 }
 
 
