@@ -21,6 +21,10 @@
 
 extern buffer ibuf, obuf, nbuf, cbuf;
 extern regexp responses[];
+extern SSL_CTX *ssl3ctx, *ssl23ctx, *tls1ctx;
+#if OPENSSL_VERSION_NUMBER >= 0x01000100fL
+extern SSL_CTX *tls11ctx, *tls12ctx;
+#endif
 
 options opts;			/* Program options. */
 environment env;		/* Environment variables. */
@@ -38,6 +42,7 @@ int
 main(int argc, char *argv[])
 {
 	int c;
+	char *cafile = NULL, *capath = NULL;
 
 	setlocale(LC_CTYPE, "");
 
@@ -47,11 +52,12 @@ main(int argc, char *argv[])
 	opts.config = NULL;
 	opts.oneline = NULL;
 	opts.debug = NULL;
+	opts.truststore = "/etc/ssl/certs";
 
 	env.home = NULL;
 	env.pathmax = -1;
 
-	while ((c = getopt(argc, argv, "Vc:d:e:il:v?")) != -1) {
+	while ((c = getopt(argc, argv, "Vc:d:e:il:t:v?")) != -1) {
 		switch (c) {
 		case 'V':
 			version();
@@ -72,6 +78,9 @@ main(int argc, char *argv[])
 		case 'l':
 			opts.log = optarg;
 			break;
+		case 't':
+			opts.truststore = optarg;
+			break;
 		case 'v':
 			opts.verbose = 1;
 			break;
@@ -100,6 +109,25 @@ main(int argc, char *argv[])
 
 	SSL_library_init();
 	SSL_load_error_strings();
+	ssl3ctx = SSL_CTX_new(SSLv3_client_method());
+	ssl23ctx = SSL_CTX_new(SSLv23_client_method());
+	tls1ctx = SSL_CTX_new(TLSv1_client_method());
+#if OPENSSL_VERSION_NUMBER >= 0x01000100fL
+	tls11ctx = SSL_CTX_new(TLSv1_1_client_method());
+	tls12ctx = SSL_CTX_new(TLSv1_2_client_method());
+#endif
+
+	if (exists_dir(opts.truststore))
+		capath = opts.truststore;
+	if (exists_file(opts.truststore))
+		cafile = opts.truststore;
+	SSL_CTX_load_verify_locations(ssl3ctx, cafile, capath);
+	SSL_CTX_load_verify_locations(ssl23ctx, cafile, capath);
+	SSL_CTX_load_verify_locations(tls1ctx, cafile, capath);
+#if OPENSSL_VERSION_NUMBER >= 0x01000100fL
+	SSL_CTX_load_verify_locations(tls11ctx, cafile, capath);
+	SSL_CTX_load_verify_locations(tls12ctx, cafile, capath);
+#endif
 
 	start_lua();
 #if LUA_VERSION_NUM < 502
@@ -118,6 +146,13 @@ main(int argc, char *argv[])
 #endif
 	stop_lua();
 
+	SSL_CTX_free(ssl3ctx);
+	SSL_CTX_free(ssl23ctx);
+	SSL_CTX_free(tls1ctx);
+#if OPENSSL_VERSION_NUMBER >= 0x01000100fL
+	SSL_CTX_free(tls11ctx);
+	SSL_CTX_free(tls12ctx);
+#endif
 	ERR_free_strings();
 
 	regexp_free(responses);
