@@ -67,6 +67,7 @@ typedef struct options {
 	char *config;		/* Configuration file. */
 	char *oneline;		/* One line of program/configuration. */
 	char *debug;		/* Debug file. */
+        char *truststore;       /* CA TrustStore. */
 } options;
 
 /* Environment variables. */
@@ -177,7 +178,7 @@ int request_delete(session *ssn, const char *mbox);
 int request_rename(session *ssn, const char *oldmbox, const char *newmbox);
 int request_subscribe(session *ssn, const char *mbox);
 int request_unsubscribe(session *ssn, const char *mbox);
-int request_idle(session *ssn);
+int request_idle(session *ssn, char **event);
 
 /*	response.c	*/
 int response_generic(session *ssn, int tag);
@@ -200,7 +201,7 @@ int response_fetchdate(session *ssn, int tag, char **date);
 int response_fetchsize(session *ssn, int tag, char **size);
 int response_fetchstructure(session *ssn, int tag, char **structure);
 int response_fetchbody(session *ssn, int tag, char **body, size_t *len);
-int response_idle(session *ssn, int tag);
+int response_idle(session *ssn, int tag, char **event);
 
 /*	signal.c	*/
 void catch_signals(void);
