@@ -68,13 +68,15 @@ send_request(session *ssn, const char *fmt,...)
 	va_start(args, fmt);
 	n = vsnprintf(obuf.data + obuf.len, obuf.size - obuf.len -
 	    strlen("\r\n") + 1, fmt, args);
+	va_end(args);
 	if (n > (int)obuf.size) {
 		buffer_check(&obuf, n);
+		va_start(args, fmt);
 		vsnprintf(obuf.data + obuf.len, obuf.size - obuf.len -
 		    strlen("\r\n") + 1, fmt, args);
+		va_end(args);
 	}
 	obuf.len = strlen(obuf.data);
-	va_end(args);
 
 	snprintf(obuf.data + obuf.len, obuf.size - obuf.len + 1, "\r\n");
 	obuf.len = strlen(obuf.data);
@@ -167,9 +169,7 @@ request_login(session **ssnptr, const char *server, const char *port, const
 		ssn->username = user;
 		ssn->password = pass;
 
-		if ((!strncasecmp(ssl, "tls1", 4) ||
-		    !strncasecmp(ssl, "ssl3", 4) ||
-		    !strncasecmp(ssl, "ssl2", 4)))
+		if (strlen(ssl) != 0)
 			ssn->sslproto = ssl;
 	} else {
 		debug("recovering connection: %s://%s@%s:%s/%s\n",
@@ -209,8 +209,8 @@ request_login(session **ssnptr, const char *server, const char *port, const
 			CHECK(t = send_request(ssn, "AUTHENTICATE CRAM-MD5"));
 			CHECK(r = response_authenticate(ssn, t, &in));
 			if (r == STATUS_CONTINUE) {
-				if ((out = auth_cram_md5(user, pass, in)) ==
-				    NULL)
+				if ((out = auth_cram_md5(ssn->username,
+				    ssn->password, in)) == NULL)
 					goto abort;
 				CHECK(send_continuation(ssn, (char *)(out),
 				    strlen((char *)(out))));
@@ -239,14 +239,16 @@ request_login(session **ssnptr, const char *server, const char *port, const
 	CHECK(t = send_request(ssn, "CAPABILITY"));
 	CHECK(response_capability(ssn, t));
 
-	if (!ssn->ns.delim && ssn->capabilities & CAPABILITY_NAMESPACE &&
+	if (ssn->capabilities & CAPABILITY_NAMESPACE &&
 	    get_option_boolean("namespace")) {
 		CHECK(t = send_request(ssn, "NAMESPACE"));
 		CHECK(response_namespace(ssn, t));
 	}
 
 	if (ssn->selected) {
-		CHECK(t = send_request(ssn, "SELECT \"%s\"", ssn->selected));
+		CHECK(t = send_request(ssn, "SELECT \"%s\"",
+		    apply_namespace(ssn->selected, ssn->ns.prefix,
+		    ssn->ns.delim)));
 		CHECK(response_select(ssn, t));
 	}
 
@@ -318,7 +320,7 @@ request_select(session *ssn, const char *mbox)
 	TRY(r = response_select(ssn, t));
 
 	if (r == STATUS_OK)
-		ssn->selected = mbox;
+		ssn->selected = xstrdup(mbox);
 	
 	return r;
 }
@@ -335,8 +337,10 @@ request_close(session *ssn)
 	TRY(t = send_request(ssn, "CLOSE"));
 	TRY(r = response_generic(ssn, t));
 
-	if (r == STATUS_OK && ssn->selected)
+	if (r == STATUS_OK && ssn->selected) {
+		xfree(ssn->selected);
 		ssn->selected = NULL;
+	}
 
 	return r;
 }
@@ -632,10 +636,14 @@ request_append(session *ssn, const char *mbox, const char *mesg, size_t
 	m = apply_namespace(mbox, ssn->ns.prefix, ssn->ns.delim);
 
 	TRY(t = send_request(ssn, "APPEND \"%s\"%s%s%s%s%s%s {%d}", m,
-		(flags ? " (" : ""), (flags ? flags : ""),
-		(flags ? ")" : ""), (date ? " \"" : ""),
-		(date ? date : ""), (date ? "\"" : ""), mesglen));
+	    (flags ? " (" : ""), (flags ? flags : ""),
+	    (flags ? ")" : ""), (date ? " \"" : ""),
+	    (date ? date : ""), (date ? "\"" : ""), mesglen));
 	TRY(r = response_continuation(ssn, t));
+	if (r == STATUS_CONTINUE) {
+		TRY(send_continuation(ssn, mesg, mesglen)); 
+		TRY(r = response_generic(ssn, t));
+	}
 
 	if (r == STATUS_TRYCREATE) {
 		TRY(t = send_request(ssn, "CREATE \"%s\"", m));
@@ -645,15 +653,14 @@ request_append(session *ssn, const char *mbox, const char *mesg, size_t
 			TRY(response_generic(ssn, t));
 		}
 		TRY(t = send_request(ssn, "APPEND \"%s\"%s%s%s%s%s%s {%d}", m,
-			(flags ? " (" : ""), (flags ? flags : ""),
-			(flags ? ")" : ""), (date ? " \"" : ""),
-			(date ? date : ""), (date ? "\"" : ""), mesglen));
+		    (flags ? " (" : ""), (flags ? flags : ""),
+		    (flags ? ")" : ""), (date ? " \"" : ""),
+		    (date ? date : ""), (date ? "\"" : ""), mesglen));
 		TRY(r = response_continuation(ssn, t));
-	}
-
-	if (r == STATUS_CONTINUE) {
-		TRY(send_continuation(ssn, mesg, mesglen)); 
-		TRY(r = response_generic(ssn, t));
+		if (r == STATUS_CONTINUE) {
+			TRY(send_continuation(ssn, mesg, mesglen)); 
+			TRY(r = response_generic(ssn, t));
+		}
 	}
 
 	return r;
@@ -752,12 +759,12 @@ request_unsubscribe(session *ssn, const char *mbox)
 
 
 int
-request_idle(session *ssn)
+request_idle(session *ssn, char **event)
 {
 	int t, r, ri;
 
 	if (!(ssn->capabilities & CAPABILITY_IDLE))
-		return -1;
+		return STATUS_BAD;
 
 	do {
 		ri = 0;
@@ -765,7 +772,7 @@ request_idle(session *ssn)
 		TRY(t = send_request(ssn, "IDLE"));
 		TRY(r = response_continuation(ssn, t));
 		if (r == STATUS_CONTINUE) {
-			TRY(ri = response_idle(ssn, t));
+			TRY(ri = response_idle(ssn, t, event));
 			TRY(send_continuation(ssn, "DONE", strlen("DONE")));
 			TRY(r = response_generic(ssn, t));
 		}
